<?php

namespace Drupal\bigin_crm;

use Drupal\Core\Url;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\bigin_crm\Rest\RestClient;

/**
 * Bigin OAuth2 authentication.
 */
class BiginAuthService {

  /**
   * Config Factory service object.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The rest client
   *
   * @var \Drupal\bigin_crm\Rest\RestClient
   */
  protected $client;

  /**
   * Constructs a new Service object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface
   *   Config Factory service object.
   * @param \Drupal\bigin_crm\Rest\RestClient
   *   The rest client.
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    RestClient $client
  ) {
    $this->configFactory = $configFactory;
    $this->client = $client;
  }

  /**
   * Generate access token.
   *
   * @param string $code
   *   Authorization code.
   *
   * @return bool
   *   Returns true if the access token could be generated.
   */
  public function generate_access_token($code) {
    $redirectUri = Url::fromRoute('bigin_crm.callback',[], ['absolute' => TRUE])->toString();
    $params = [
      'client_id' => $this->setting('client_id'),
      'client_secret' => $this->setting('client_secret'),
      'code' => $code,
      'grant_type' => 'authorization_code',
      'redirect_uri' => $redirectUri
    ];

    $url = $this->url_account() . '/oauth/v2/token';
    $response = $this->client->api_call($url, $params, [], 'POST');
    // save token
    if(!empty($response->access_token)) {
      $this->configFactory->getEditable('bigin_crm.settings')
      ->set('access_token', $response->access_token)
      ->set('refresh_token', $response->refresh_token)
      ->save();
    }
    return empty($response->access_token) ? FALSE : TRUE;
  }

  /**
   * Get access token.
   *
   * @return string
   *   Token.
   */
  public function access_token() {
    return $this->setting('access_token') ?? '';
  }

  /**
   * Get new access token using refresh token.
   *
   * @return bool
   *   Returns TRUE if the access token could be generated.
   */
  public function refresh_token() {
    // Request Parameters
    $params = [
      'client_id' => $this->setting('client_id'),
      'client_secret' => $this->setting('client_secret'),
      'refresh_token' => $this->setting('refresh_token'),
      'grant_type' => 'refresh_token',
    ];

    $url = $this->url_account() . '/oauth/v2/token';
    $response = $this->client->api_call($url, $params, [], 'POST');

    if(!empty($response->access_token)) {
      // Save token
      $this->configFactory->getEditable('bigin_crm.settings')
        ->set('access_token', $response->access_token)
        ->save();
    }
    return empty($response->access_token) ? FALSE : TRUE;
  }

  /**
   * Revoke refresh tokens.
   *
   * @return string
   * Returns the response from the api
   */
  public function revoke_token() {
    $params = [
      'token' => $this->access_token(),
    ];
    $url = $this->url_account() . '/oauth/v2/token/revoke';
    $response = $this->client->api_call($url, $params, [], 'POST');

    if(!empty($response->status) && $response->status == 'success') {
         $this->configFactory->getEditable('bigin_crm.settings')
        ->set('access_token', '')
        ->set('refresh_token', '')
        ->save();
    }
    return $response->status ?? 'Error';
  }

  /**
   * Get domain-specific accounts URL to generate access and refresh token.
   *
   * @return string
   *   The URL.
   */
  public function url_account() {
    $accountsArray = [
      'com' => 'https://accounts.zoho.com',
      'eu' => 'https://accounts.zoho.eu',
      'cn' => 'https://accounts.zoho.com.cn',
      'in' => 'https://accounts.zoho.in',
    ];
    return $accountsArray[$this->setting('domain')];
  }

  /**
   * Get domain specific accounts url for rest api.
   *
   * @return string
   *   The URL.
   */
  public function url_api() {
    $apiArray = [
      'com' => 'https://www.zohoapis.com',
      'eu' => 'https://www.zohoapis.eu',
      'cn' => 'https://www.zohoapis.com.cn',
      'in' => 'https://www.zohoapis.in',
    ];
    return $apiArray[$this->setting('domain')];
  }

  /**
   * Access the settings of this module.
   *
   * @param string $key
   *   The key of the configuration.
   *
   * @return \Drupal\Core\Config\ImmutableConfig
   *   The value of the configuration item requested.
   */
  protected function setting($key) {
    return $this->configFactory->get('bigin_crm.settings')->get($key);
  }
}
